
# q-expansion of newform 3300.2.a.w, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3300
weight = 2
poly_data = [-3, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [1, 0], [0, 0], [-1, -1], [1, 0], [1, 1], [-3, -1], [0, -2], [0, 0], [8, 0], [2, 2], [-4, 2], [8, 0], [7, -1], [-2, 2], [-2, 0], [8, 0], [0, 2], [4, 0], [0, -4], [3, -1], [-4, -2], [7, 1], [6, 0], [-2, 4], [-2, 2], [6, -2], [-5, -3], [-10, 0], [8, -2], [5, 1], [-14, 2], [6, 0], [2, 4], [-2, -2], [-12, -2], [4, -2], [-2, 2], [9, 3], [-11, 3], [12, 0], [16, 2], [4, 4], [11, 3], [17, -1], [4, -4], [-16, -2], [-2, -2], [5, 3], [6, -4], [7, 1], [14, -2], [-8, -2], [4, -4], [16, 2], [-1, 5], [-6, -4], [-12, 2], [-7, -7], [0, 0], [-1, -1], [-9, -3], [-5, -1], [16, 4], [-16, 2], [-4, -2], [-14, 2], [3, 7], [11, 5], [-16, 2], [26, 0], [26, -2], [-14, 2], [-1, -1], [-4, -4], [-26, 2], [2, 8], [-14, -4], [6, 4], [-2, 0], [12, 0], [-12, -2], [-16, 4], [6, -4], [-10, 0], [-18, -2], [-10, 8], [1, 1], [-2, -2], [6, -2], [4, 0], [-22, 2], [-20, 4], [-20, 4], [-4, 0], [-15, 3], [-14, -4], [6, -4], [-3, -7], [18, -4], [25, -3], [25, -5], [9, 3], [-6, -6], [-2, 4], [-36, 2], [8, -4], [5, -5], [-24, 0], [-16, 2], [-23, 5], [11, 3], [32, -2], [2, -6], [8, 8], [-10, 8], [-22, 6], [-38, -2], [-12, 2], [-36, 0], [2, 12], [15, -1], [-9, 1], [38, -2], [-28, 4], [32, 4], [-16, 6], [-12, -4], [0, 8], [11, -1], [-2, 12], [-41, 1], [8, 0], [20, -2], [6, 6], [-36, -2], [12, 10], [-21, -5], [-4, -10], [-12, 0], [-22, 4], [16, -8], [-34, -2], [-1, 1], [28, -6], [16, 4], [15, 7], [-7, -5], [50, 2], [-18, -6], [-1, 7], [-2, 8], [8, -4], [5, -5], [-2, 10], [4, 4], [-26, 0], [-2, 8], [-39, 5], [46, 2], [-18, 6], [3, 9], [-15, -3], [12, 0], [30, 0], [-2, -14], [-14, -6], [3, -5], [-40, 6], [-18, 0], [52, 0], [0, 6], [14, 2], [-3, -11], [-14, -2], [42, -4], [-6, -8], [6, 0], [13, 1], [34, -8], [24, 8], [-36, 0], [-12, -10], [3, 9], [22, 2], [-34, 4], [3, -1], [-10, -6], [30, 8], [12, 0], [8, 14], [-1, 9], [-20, 0], [2, 0], [-35, 7], [-36, 6], [36, -2], [46, -4], [-1, -11], [7, -7], [-4, 0], [22, 4], [14, -4], [-16, 6], [-20, 8], [-56, 2], [20, 0], [57, 3], [32, -4], [10, -10], [1, -3], [-30, 8], [0, -8], [0, -12], [-10, 10], [-22, -12], [-1, -13], [-2, -14], [-52, -4], [12, -6], [-52, 2], [16, -6], [22, 0], [2, 18], [19, -7], [-6, 8], [46, 0], [-56, -4], [33, -7], [6, 2], [42, -4], [18, -4], [-30, 8], [-40, -4], [68, 0], [-1, 9], [24, -10], [7, -11], [12, -4], [-12, -12], [46, 6], [-4, -10], [-36, 4], [0, 2], [-1, -7], [-30, -2], [8, -8], [-16, 8], [26, 0], [43, 5], [45, 5], [14, 10], [52, -4], [6, -16], [-15, 3], [24, 8], [-28, -6], [-19, 5], [-8, 2], [-35, -7], [-13, 7], [21, -1], [0, -2], [-11, -7], [-2, 4], [8, 12], [54, 4], [-18, 0], [23, -5], [-13, 5], [-14, 0], [26, -10], [12, 10], [50, 4], [-31, 1], [28, 4], [-30, 10], [40, -6], [20, -2], [-6, 18], [-33, 5], [42, 2], [-53, 5], [6, -12], [33, 5], [32, 0], [18, -8], [3, 13], [42, 2], [-18, -6], [-22, -8], [14, -10], [-21, 13], [10, -6], [5, 9], [34, 10], [4, 4], [-60, -6], [8, 8], [11, 15], [-23, -11], [-35, -1], [-38, 0], [36, 0], [-6, -10], [-24, 10], [-58, -2], [-6, 4], [-16, 8], [47, -9], [-15, -13], [6, -20], [14, -14], [-20, -16], [25, -13], [-8, -18], [-20, -16], [14, 10], [44, 2], [-6, 10], [24, -2], [4, -6], [20, -8], [-4, 4], [-9, -15], [-16, 6], [30, -14], [-56, 4], [15, -3], [25, -17], [-12, -14], [22, 12], [-40, -6], [-7, -9], [-44, 6], [-30, -6], [66, 0], [11, 13], [-40, 6], [-17, -9], [22, 8], [14, 12], [-62, 2], [10, -2], [-48, 6], [54, 10], [22, 0], [54, 2], [-20, -4], [-80, -2], [18, -4], [-14, -12], [-10, 4], [9, 5], [-44, 6], [-35, 7], [-12, 16], [2, -18], [-33, 5], [-32, 0], [21, 1], [18, 6], [24, -8], [-18, 10], [56, -4], [-36, -14], [-3, -9], [-5, 3], [-4, 22], [24, 16], [-22, 10], [-33, 9], [-18, -22], [30, -8], [16, -14], [20, 8], [46, -2], [-67, 9], [-2, -26], [45, 13], [18, 0], [-48, 10], [65, -3], [21, -1], [-16, -22], [48, -8], [44, 12], [-10, -8], [9, -19], [-28, 4], [16, 2], [-14, 4], [-28, -4], [36, 8], [4, 0], [45, 5], [-26, 12], [22, -20], [32, 0], [12, -12], [-30, 8], [16, -2], [39, 7], [-20, 22], [-46, 2], [-52, -10], [-18, 8], [34, 14], [-24, -12], [-28, -4], [-39, -11], [-41, 5], [-48, -12], [18, 8], [-65, 3], [50, 4], [44, 8], [-26, -18], [18, 8], [13, 7], [6, 12], [33, 13], [12, 20], [42, 6], [2, 12], [-24, 6], [-64, 4], [-38, 14], [-12, -4], [22, -10]]
